@@ -1,8 +1,10 @@
-import os, time, sys, threading, ctypes, signal, keyboard
-from typing import Any, Optional, Union, overload
+import inspect
+import os, time, sys, threading, ctypes, signal, keyboard, asyncio
+from typing import Any, Optional, Union
 from types import FunctionType
 from .__workerhelp import help_msg
 
+
 class ThreadWorker():
     """
     ThreadWorker class -> worker
@@ -11,11 +13,11 @@ class ThreadWorker():
     """
 
     def __init__(self, func, name="", on_abort=None, enable_keyboard_interrupt=True):
-        if name:
-            if name in ThreadWorkerManager.allWorkers.keys():
-                name = name+str(ThreadWorkerManager.counts)
-        else:
-            name = 'worker'+str(ThreadWorkerManager.counts)
+        if not name: name = "worker"
+        if name in ThreadWorkerManager.allWorkers:
+            name = name+str(ThreadWorkerManager.counts)
+        ThreadWorkerManager.allWorkers[name] = self
+        ThreadWorkerManager.counts += 1
         self.name = name
         self.id = ThreadWorkerManager.counts
         self.func = None
@@ -32,16 +34,27 @@ class ThreadWorker():
         self.__work_time = 0
     
     def __call__(self, *args: Any, **kwargs: Any) -> Any:
-        self.func = lambda: self.rawfunc(*args, **kwargs)
-        ThreadWorkerManager.allWorkers[self.name] = self
-        ThreadWorkerManager.counts += 1
+        if inspect.iscoroutinefunction(self.rawfunc):
+            async def work_func():
+                return await self.rawfunc(*args, **kwargs)
+            self.func = work_func
+        else:
+            self.func = lambda: self.rawfunc(*args, **kwargs)
         self.work()
         return self
 
+    def __get_working_time(self):
+        t = time.perf_counter()-self.start_time
+        lst = len(str(t).split(".")[0])
+        if lst < 10:
+            return round(t, 10-lst)
+        else:
+            return int(t)
+
     @property
     def work_time(self):
         if self.thread and not self.finished:
-            self.__work_time = round(time.perf_counter()-self.start_time, 2)
+            self.__work_time = self.__get_working_time()
         return self.__work_time
 
     @property       
@@ -65,8 +78,27 @@ class ThreadWorker():
                 self.__ret = self.rawfunc()
             self.__finishStat = True
         finally:
+            self.__work_time = self.__get_working_time()
+            if not self.__finishStat:
+                self.__finishStat = True
+                try:
+                    if self.aborted_by_kbInterrupt: print(self.id_mark,"KeyboardInterrupt", flush=True)
+                    if self.on_abort:
+                        self.on_abort()
+                except Exception as e:                      
+                    print(self.id_mark,"OnAbortError",str(type(self.on_abort)), flush=True)
+    
+    async def __aexecute(self):
+        try:
+            self.__finishStat = False
+            if self.func:
+                self.__ret = await self.func()
+            else:
+                self.__ret = await self.rawfunc()
+            self.__finishStat = True
+        finally:
+            self.__work_time = self.__get_working_time()
             if not self.__finishStat:
-                self.__work_time = round(time.perf_counter()-self.start_time, 2)
                 self.__finishStat = True
                 try:
                     if self.aborted_by_kbInterrupt: print(self.id_mark,"KeyboardInterrupt", flush=True)
@@ -76,8 +108,12 @@ class ThreadWorker():
                     print(self.id_mark,"OnAbortError",str(type(self.on_abort)), flush=True)
 
     def work(self):
-        self.thread = threading.Thread(target=self.__execute)
-        self.thread.start()
+        if inspect.iscoroutinefunction(self.rawfunc):
+            self.thread = threading.Thread(target=lambda: asyncio.run(self.__aexecute()))
+            self.thread.start()
+        else:
+            self.thread = threading.Thread(target=self.__execute)
+            self.thread.start()
 
     def interrupt(self):
         if self.enable_keyboard_interrupt:
@@ -141,27 +177,26 @@ class StaticThreadWorkerManager():
                 raise BaseException(e)
         if margs:
             if type(margs[0]) == FunctionType:
+                assert not inspect.iscoroutinefunction(margs[0]), "please use `async_worker` instead for coroutine function"
                 def register(*args,**kargs):
-                    workerName = 'worker'+str(ThreadWorkerManager.counts)
-                    ThreadWorkerManager.allWorkers[workerName] = ThreadWorker(
+                    w = ThreadWorker(
 						lambda: margs[0](*args,**kargs),
-						workerName, 
+						"worker", 
 						on_abort, interrupt)
-                    ThreadWorkerManager.allWorkers[workerName].work()
-                    ThreadWorkerManager.counts += 1
-                    return ThreadWorkerManager.allWorkers[workerName]
+                    w.work()
+                    return w
                 return register
             elif type(margs[0]) == str:
                 def applying(func):
+                    assert not inspect.iscoroutinefunction(func), "please use `async_worker` instead for coroutine function"
                     def register(*args,**kargs):
-                        workerName = margs[0] if margs[0] not in ThreadWorkerManager.allWorkers.keys() else margs[0]+str(ThreadWorkerManager.counts)
-                        ThreadWorkerManager.allWorkers[workerName] = ThreadWorker(
-							lambda: func(*args,**kargs),
-							workerName,
-							on_abort, interrupt)
-                        ThreadWorkerManager.allWorkers[workerName].work()
-                        ThreadWorkerManager.counts += 1
-                        return ThreadWorkerManager.allWorkers[workerName]
+                        workerName = margs[0]
+                        w = ThreadWorker(
+                            lambda: func(*args,**kargs),
+                            workerName,
+                            on_abort, interrupt)
+                        w.work()
+                        return w                        
                     return register
                 return applying
             else:
@@ -169,6 +204,54 @@ class StaticThreadWorkerManager():
         else:
             return ThreadWorkerManager._workerDefinitionError()
     
+    @staticmethod
+    def create_async_worker(*margs,**kargs):
+        on_abort = None
+        interrupt = True		
+        if kargs:
+            if "on_abort" in kargs: 
+                on_abort = kargs["on_abort"]
+            if "keyboard_interrupt" in kargs: 
+                interrupt = bool(kargs["keyboard_interrupt"])
+            if "interrupt" in kargs: 
+                interrupt = bool(kargs["interrupt"])
+            if not margs: 
+                e = "Error: on_abort requires worker name on decorator\nPlease read ThreadWorkerManager.help()"
+                raise BaseException(e)
+        if margs:
+            if type(margs[0]) == FunctionType:
+                assert inspect.iscoroutinefunction(margs[0]), "please use `worker` instead for non-coroutine function"
+                async def register(*args,**kargs):
+                    async def worker_func():
+                        return await args[0](*args,**kargs)
+                    w = ThreadWorker(
+						worker_func,
+						"worker", 
+						on_abort, interrupt)
+                    w.work()
+                    return w
+                return register
+            elif type(margs[0]) == str:
+                def applying(func):
+                    assert inspect.iscoroutinefunction(func), "please use `worker` instead for non-coroutine function"
+                    async def register(*args,**kargs):
+                        workerName = margs[0]
+                        async def worker_func():
+                            return await func(*args,**kargs)
+                        w = ThreadWorker(
+                            worker_func,
+                            workerName,
+                            on_abort, interrupt)
+                        w.work()
+                        return w
+                    return register
+                return applying
+            else:
+                return ThreadWorkerManager._workerDefinitionError()
+        else:
+            return ThreadWorkerManager._workerDefinitionError()
+    
+
     @staticmethod
     def _workerDefinitionError():
         e = "Error: Worker Decorator function or method! Please read ThreadWorkerManager.help()"
@@ -178,9 +261,9 @@ class StaticThreadWorkerManager():
     def run_as_worker(
             target: FunctionType,
             *function_args,
-            worker_name: Optional[str] = "",
-            worker_on_abort: Optional[FunctionType] = None,
-            keyboard_interrupt: bool =True,
+            _worker_name: Optional[str] = "",
+            _worker_on_abort: Optional[FunctionType] = None,
+            _keyboard_interrupt: bool =True,
             args: Optional[Union[list, tuple]] = (),
             kargs: Optional[dict] = {},
             **function_kargs
@@ -190,14 +273,29 @@ class StaticThreadWorkerManager():
         """
         args = list(args)+list(function_args)
         kargs.update(function_kargs)
-        @worker(worker_name,on_abort=worker_on_abort,keyboard_interrupt=bool(keyboard_interrupt))
+        @worker(_worker_name,on_abort=_worker_on_abort,keyboard_interrupt=_keyboard_interrupt)
         def runFunction():
             return target(*args,**kargs)
         return runFunction()
 
     @staticmethod
-    def wait(*args):
-        for i in args: i.wait()
+    def wait(*workers: ThreadWorker, wait_all: bool = False):
+        """
+        wait defined workers to be done
+        """
+        if wait_all: workers = ThreadWorkerManager.allWorkers.values()
+        for w in workers: w.wait()
+    
+    @staticmethod
+    def await_workers(*workers: ThreadWorker, await_all: bool = False) -> dict:
+        """
+        await defined workers to be done and return a dict of results
+        """
+        if await_all: workers = ThreadWorkerManager.allWorkers.values()
+        res = {}
+        for w in workers:
+            res[w.name] = w.await_worker()
+        return res
 
     @staticmethod
     def list(active_only=False):
@@ -378,7 +476,7 @@ def worker(
         *args,
         name: Optional[str] = "",
         on_abort: Optional[FunctionType] = None,
-        enable_keyboard_interrupt: Optional[bool] = True,
+        keyboard_interrupt: Optional[bool] = True,
         **kargs
     ):
     """
@@ -395,15 +493,56 @@ def worker(
     """
     if args:
         if type(args[0]) == FunctionType:
-            return ThreadWorker(args[0], name, on_abort, enable_keyboard_interrupt)
+            assert not inspect.iscoroutinefunction(args[0]), "please use `async_worker` instead for coroutine function"
+            def register(*dargs,**dkargs):
+                w = ThreadWorker(lambda: args[0](*dargs,**dkargs), name, on_abort, keyboard_interrupt)
+                w.work()
+                return w
+            return register
         elif type(args[0]) == str:
-            return ThreadWorkerManager.create_worker(args[0], *args[1:], on_abort=on_abort, keyboard_interrupt=enable_keyboard_interrupt, **kargs)
+            name = args[0]
+            return ThreadWorkerManager.create_worker(name, *args[1:], on_abort=on_abort, keyboard_interrupt=keyboard_interrupt, **kargs)
     else:
-        return ThreadWorkerManager.create_worker(name, on_abort=on_abort, keyboard_interrupt=enable_keyboard_interrupt, **kargs)
+        return ThreadWorkerManager.create_worker(name, on_abort=on_abort, keyboard_interrupt=keyboard_interrupt, **kargs)
+
+def async_worker(
+        *args,
+        name: Optional[str] = "",
+        on_abort: Optional[FunctionType] = None,
+        keyboard_interrupt: Optional[bool] = True,
+        **kargs
+    ):
+    """
+    async_worker(function) -> threaded-corotine-function
+
+    An asynchronous worker decorator.
+    Turn a main-thread coroutine function into a background-thread coroutine function automatically
+
+    Usage Example:
+    - @async_worker
+    - @async_worker("cool")
+    - @async_worker(name="looping backapp", keyboard_interrupt=True)
+    - @async_worker(keyboard_interrupt=True, on_abort: lambda: print("its over"))
+    """
+    if args:
+        if type(args[0]) == FunctionType:
+            assert inspect.iscoroutinefunction(args[0]), "please use `worker` instead for non-coroutine function"
+            async def register(*dargs,**dkargs):
+                async def worker_func():
+                    return await args[0](*dargs,**dkargs)
+                w = ThreadWorker(worker_func, name, on_abort, keyboard_interrupt)
+                w.work()
+                return w
+            return register
+        elif type(args[0]) == str:
+            name = args[0]
+            return ThreadWorkerManager.create_async_worker(name, *args[1:], on_abort=on_abort, keyboard_interrupt=keyboard_interrupt, **kargs)
+    else:
+        return ThreadWorkerManager.create_async_worker(name, on_abort=on_abort, keyboard_interrupt=keyboard_interrupt, **kargs)
 
 ### SHORTCUT ###
 ThreadWorkerManager = StaticThreadWorkerManager()
-run_as_Worker = ThreadWorkerManager.run_as_worker
+run_as_worker = ThreadWorkerManager.run_as_worker
 abort_worker = ThreadWorkerManager.abort_worker
 abort_all_worker = ThreadWorkerManager.abort_all_worker
 abort_thread = ThreadWorkerManager.abort_thread
